#!/usr/bin/perl

############################################
##                                        ##
##               CounterLog               ##
##           by Darryl Burgdorf           ##
##       (e-mail burgdorf@awsd.com)       ##
##                                        ##
##             version:  1.11             ##
##         last modified:  3/9/98         ##
##           copyright (c) 1998           ##
##                                        ##
##    latest version is available from    ##
##        http://awsd.com/scripts/        ##
##                                        ##
############################################

# COPYRIGHT NOTICE:
#
# Copyright 1998 Darryl C. Burgdorf.  All Rights Reserved.
#
# This program may be used and modified free of charge by anyone, so
# long as this copyright notice and the header above remain intact.  By
# using this program you agree to indemnify Darryl C. Burgdorf from any
# liability.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  Obtain permission before redistributing this
# program over the Internet or in any other medium.  In all cases
# copyright and header must remain intact.

# DESCRIPTION:
#
# CounterLog monitors accesses to any pages which contain an SSI code
# referencing it.  It can create NCSA-format agent and referer logs,
# which can be handy if your server doesn't happen to already provide
# you with that information.  It can also be used to put text-based
# access counters on some or all of your pages.  (Requires SSI.)

# VERSION HISTORY:
#
# 1.11  03/09/98  Corrected new glitch in URL processing
#                 Made agent & referer logs optional
# 1.10  01/23/98  Added "today's" hits to counter display
#                 Yet a few more small tweaks
# 1.02  08/06/97  A few small tweaks
# 1.01  05/05/97  Fixed minor bug in file locking
#                 Minor format changes
# 1.00  02/25/97  Initial "public" release

#########
# SETUP #
#########

# Any page you want monitored should contain somewhere the following
# SSI code:
#
# <!--#exec cgi="/cgi-bin/counterlog.pl"-->
#
# (If you've put the CounterLog script somewhere other than in your
# cgi-bin directory, the location should be revised accordingly.)
#
# If you're putting a counter on the page, of course, the SSI code
# should be placed on the page where you want the counter to appear.
# Otherwise, it makes no difference at all where you put it.

# The following variables define the location of the three log files.
# Make sure that the files actually exist and that they are set world-
# writable!

$AccessFile = "/usr/www/users/foo/count_access";
$AgentFile = "/usr/www/users/foo/count_agent";
$RefererFile = "/usr/www/users/foo/count_referer";

# If $IncludeDomain is set to "1" the access and referer logs will
# include the domain of the local pages accessed.  This can be handy
# if you have several different virtual domains on the system, and want
# to be able to keep track of accesses to pages with the same names
# (e.g., "index.html") under each domain.  Setting it to "0" will
# prevent the domain from being listed.

$IncludeDomain = 1;

# If $IgnoreIntRefs is set to "1" the referer log will *not* list
# references from another page within the same domain.  (This works
# regardless of the setting of $IncludeDomain.)  This helps to keep
# the size of the referer log down.  Set this to "0" if you need to
# know how often people are coming to a given page from each of
# several other pages on the same site, but set it to "1" if you're
# really only interested in knowing which outside pages link to yours.

$IgnoreIntRefs = 1;

# If you want to ignore accesses from specific domain names and/or
# IP numbers, list them here:

# @IgnoreIP = ("");

# By default, the CounterLog script prints nothing on a page which
# accesses it.  If you want it to print a counter number on the page,
# list it here as it appears on the access list:

@PlainCounter = ("foo.com/file1.html","foo.com/file2.html");

# If you want a particular page to feature an ordinalized counter
# (e.g., 1st, 2nd, 3rd, etc.) rather than a plain counter, list it
# here:

@OrdCounter = ("");

###############
# ACCESS FILE #
###############

$UseLocking = 1;

print "Content-type: text/html\n\n";

open (COUNT,"+<$AccessFile");

$lockerror = &LockFile(COUNT);

if ($lockerror) {
	print "[Timeout Locking Access Log]";
	exit(0);
}

if (defined $ENV{'DOCUMENT_URI'}) {
	$doc_uri = $ENV{'DOCUMENT_URI'};
	if (defined $ENV{'SERVER_NAME'}) {
		$server = $ENV{'SERVER_NAME'};
		$server =~ s/www\.//i;
		if ($IncludeDomain) {
			$doc_uri = $server . $doc_uri;
		}
	}
}
else { $doc_uri = ""; }

$doc_uri =~ s/([^:])\/\//$1\//g;
$doc_uri =~ s/\/$//;

$location = tell COUNT;

while ($line = <COUNT>) {
	if (($acc,$day,$dayacc,$uri) = ($line =~ /^(\d+) (\d+) (\d+) '(\S+)'$/)) {
		if ($uri eq $doc_uri) {
			last;
		}
	}
	last if ($uri eq $doc_uri);
	$location = tell COUNT;
	$acc = 0;
	$dayacc = 0;
}

$acc += 1;

$time = time;
($mday,$mon,$year) = (localtime($time))[3,4,5];
$year += 1900;
if ($mday < 10) { $mday = "0".$mday; }
$mon += 1;
if ($mon < 10) { $mon = "0".$mon; }
$today = $year.$mon.$mday;

unless ($day eq $today) {
	$dayacc = 0;
}

$dayacc += 1;

foreach $key (@PlainCounter) {
	if ($doc_uri eq $key) {
		$printacc = &commas($acc);
		$printdayacc = &commas($dayacc);
		last;
	}
}

foreach $key (@OrdCounter) {
	if ($doc_uri eq $key) {
		$printacc = &commas($acc) . &ordinalize($acc);
		$printdayacc = &commas($dayacc) . &ordinalize($dayacc);
		last;
	}
}

if ($printacc) { print "$printacc ($printdayacc today)"; }

$ignore = 0;

if ((defined $ENV{"REMOTE_ADDR"}) && (defined @IgnoreIP)) {
	$ignore = grep($ENV{"REMOTE_ADDR"} =~ /$_/, @IgnoreIP);
}

if ($doc_uri && !$ignore) {
	seek(COUNT, $location, 0);
	$longacc = sprintf("%010.10d", $acc);
	$longdayacc = sprintf("%010.10d", $dayacc);
	print COUNT "$longacc $today $longdayacc '$doc_uri'\n";
}

&UnlockFile(COUNT);

close COUNT;

##############
# AGENT FILE #
##############

if ($AgentFile) {
	open (COUNT,">>$AgentFile");
	$lockerror = &LockFile(COUNT);
	if ($lockerror) {
		print "[Timeout Locking Agent Log]";
		exit(0);
	}
	if (defined $ENV{'HTTP_USER_AGENT'}) { 
		$doc_agent = $ENV{'HTTP_USER_AGENT'};
	}
	else { $doc_agent = ""; }
	if ($doc_agent && !($ignore)) {
		print COUNT "$doc_agent\n";
	}
	&UnlockFile(COUNT);
	close COUNT;
}

################
# REFERER FILE #
################

if ($RefererFile) {
	open (COUNT,">>$RefererFile");
	$lockerror = &LockFile(COUNT);
	if ($lockerror) {
		print "[Timeout Locking Referer Log]";
		exit(0);
	}
	if (defined $ENV{'HTTP_REFERER'}) {
		$doc_referer = $ENV{'HTTP_REFERER'};
	}
	else { $doc_referer = ""; }
	$doc_referer =~ s/([^:])\/\//$1\//g;
	$doc_referer =~ s/\/$//;
	if ($doc_referer && $doc_uri && !($ignore)) {
		unless ($IgnoreIntRefs && ($doc_referer =~ m#$server#oi)) {
			print COUNT "$doc_referer -> $doc_uri\n";
		}
	}
	&UnlockFile(COUNT);
	close COUNT;
}

###############
# SUBROUTINES #
###############

sub LockFile {
	local(*FILE) = @_;
	local($TrysLeft) = 10;
	if ($UseLocking) {
		while ($TrysLeft--) {
			$lockresult = eval("flock(FILE,6)");
			if ($@) {
				$UseLocking = 0;
				last;
			}
			if (!$lockresult) {
				select(undef,undef,undef,0.1);
			}
			else {
				last;
			}
		}
	}
	if ($TrysLeft >= 0) {
		return 0;
	}
	else {
		return -1;
	}
}

sub UnlockFile {
	local(*FILE) = @_;
	if ($UseLocking) {
		flock(FILE,8);
	}
}

sub ordinalize {
	local($count) = @_;
	local($last, $last2);
	$last2 = $count % 100;
	$last = $count % 10;
	if ($last2 < 10 || $last2 > 13) {
		return "st" if $last == 1;
		return "nd" if $last == 2;
		return "rd" if $last == 3;
	}
	return "th";
}

sub commas {
	local($_)=@_;
	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

